package org.eclipse.me.emf.notification.test;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.epsilon.emc.emf.EmfModel;
import org.eclipse.epsilon.eol.exceptions.models.EolModelLoadingException;

public class ElementMonitor {
	private Resource resource;
	private EmfModel sourceModel;
	private String modelFile = "src/model/test-model.ecore";
	private String modelName = "gmfgraph";

	public static void main(String[] args) {
		
		ElementMonitor monitor = new ElementMonitor();
		try {
			monitor.loadModel();
		} catch (EolModelLoadingException e) {
			e.printStackTrace();
		}
		
		monitor.createResource();
		monitor.modifyModel();
	}

	public Resource createResource() {
		resource = sourceModel.getResource();
		resource.setTrackingModification(true);
		
		EContentAdapter adapter = new EContentAdapter() {
			
			public Notifier getTarget() {
				return null;
			}
			
			public boolean isAdapterForType(Object type) {
				return false;
			}
			
			public void notifyChanged(Notification notification) {
				super.notifyChanged(notification);
				processNotification(notification);
			}
			
			public void setTarget(Notifier newTarget) {
				
			}
			
		};
		
		resource.eAdapters().add(adapter);
		for(EObject obj : resource.getContents()) {
			obj.eAdapters().add(adapter);
			for(EObject o : obj.eContents()) {
				o.eAdapters().add(adapter);
			}
		}
		
		return resource;
	}
	
	
	protected void processNotification(Notification notification) {
		
		if(!(notification.getNotifier() instanceof Resource)) {
			EObject notifier = (EObject) notification.getNotifier();
			
			System.out.println("Object of id: " + sourceModel.getElementId(notifier) + " changed from: --- " + notification.getOldValue() + " to: --- " + notification.getNewValue());
			
			System.out.println(EcoreUtil.getID(notifier));
			
		}
		
	}
	
	private void loadModel() throws EolModelLoadingException {
		sourceModel = new EmfModel();
		sourceModel.setModelFile(modelFile);
		sourceModel.setName(modelName);
		sourceModel.load();
	}
	
	public void modifyModel() {
		EcoreUtil.delete(resource.getContents().get(0));
	}
	
}

